import React, { useState, useEffect, useCallback } from "react";
import { useNavigate } from "react-router-dom";
import { Button, Typography, Grid, Tooltip } from "@mui/material";
import { useUndeployInDb } from "../../../api/SwaggerApi";
import { SimpleAccordion, ConfirmationDialog } from "@ess-ics/ce-ui-common";
import Alert from "@mui/material/Alert";
import AccessControl from "../../auth/AccessControl";
import { IocActiveDeployment } from "../../../api/DataTypes";

export default function AdministerUndeployment({ ioc, buttonDisabled }) {
  const navigate = useNavigate();

  function onError(message) {
    setError(message);
  }

  // for the dialog
  const [error, setError] = useState();
  const [open, setOpen] = useState(false);

  const [response, undeployIOC] = useUndeployInDb(ioc.id, onError);

  const onClose = useCallback(() => {
    setOpen(false);
  }, []);

  const onConfirm = useCallback(() => {
    undeployIOC();
  }, [undeployIOC]);

  useEffect(() => {
    if (response) {
      navigate(-1);
    }
  }, [response, navigate]);

  const failedDeployment = new IocActiveDeployment(
    ioc.activeDeployment
  ).failedDeployment();

  let disabledButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledButtonTitle =
      "There is an ongoing operation, you cannot 'undeploy' the IOC right now";
  } else {
    if (!failedDeployment) {
      disabledButtonTitle =
        "IOC doesn't have failed (un)deployment, you cannot 'undeploy' it";
    }
  }

  return (
    <AccessControl
      allowedRoles={["DeploymentToolAdmin"]}
      renderNoAccess={() => <></>}
    >
      <>
        <ConfirmationDialog
          title="Administratively 'undeploy' IOC"
          description={
            <>
              <Typography component="span">
                Are you sure want to administer IOC as undeployed
                <Typography
                  fontFamily="monospace"
                  component="span"
                >
                  {" "}
                  {ioc.namingName}
                </Typography>{" "}
                ?
              </Typography>
            </>
          }
          confirmText="Undeploy IOC"
          cancelText="Cancel"
          open={open}
          onClose={onClose}
          onConfirm={onConfirm}
        />

        <SimpleAccordion
          summary="Change deployment status to not deployed (e.g. if IOC has been manually removed)"
          defaultExpanded
        >
          <Grid
            container
            spacing={1}
          >
            {error ? (
              <Grid
                item
                xs={12}
              >
                <Alert severity="error">{error}</Alert>
              </Grid>
            ) : (
              <></>
            )}
            <Grid
              item
              xs={12}
            >
              <Tooltip title={disabledButtonTitle}>
                <span>
                  <Button
                    onClick={() => setOpen(true)}
                    disabled={
                      buttonDisabled ||
                      ioc.operationInProgress ||
                      !failedDeployment
                    }
                    color="error"
                    variant="contained"
                  >
                    SET DEPLOYMENT STATE TO NOT DEPLOYED
                  </Button>
                </span>
              </Tooltip>
            </Grid>
          </Grid>
        </SimpleAccordion>
      </>
    </AccessControl>
  );
}
